#!/usr/bin/env ts-node-to
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var shellQuote = tslib_1.__importStar(require("./shell-quote"));
var binary_split_1 = tslib_1.__importDefault(require("binary-split"));
var fs_1 = tslib_1.__importDefault(require("fs"));
var assert_1 = tslib_1.__importDefault(require("assert"));
require('source-map-support').install();
/**
 * Read a binary stream, split into chunks on a delimiter, as a
 */
var ChunkReader = /** @class */ (function () {
    function ChunkReader(readable, delimeter) {
        if (delimeter === void 0) { delimeter = Buffer.from([0]); }
        this.readable = readable;
        this.delimeter = delimeter;
    }
    ChunkReader.prototype.start = function () {
        this.splitStream = binary_split_1.default(this.delimeter);
        this.readable.pipe(this.splitStream);
    };
    /** TODO implement returning when the stream is closed */
    ChunkReader.prototype.getChunks = function () {
        return tslib_1.__asyncGenerator(this, arguments, function getChunks_1() {
            var stream, DONE, resFns, promises, subscriptions, line;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        stream = this.splitStream;
                        DONE = {};
                        resFns = [];
                        promises = [];
                        promises.unshift(new Promise(function (res, rej) { resFns.unshift({ res: res, rej: rej }); }));
                        subscriptions = new DisposableEventListener(stream);
                        subscriptions.on('data', function (line) {
                            resFns[0].res(line);
                            promises.unshift(new Promise(function (res, rej) { resFns.unshift({ res: res, rej: rej }); }));
                        });
                        subscriptions.on('end', function () {
                            resFns[0].res(DONE);
                        });
                        subscriptions.on('error', function (error) {
                            resFns[0].rej(error);
                        });
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, , 11, 12]);
                        _a.label = 2;
                    case 2:
                        if (!true) return [3 /*break*/, 10];
                        return [4 /*yield*/, tslib_1.__await(promises[promises.length - 1])];
                    case 3:
                        line = _a.sent();
                        if (!(line === DONE)) return [3 /*break*/, 5];
                        return [4 /*yield*/, tslib_1.__await(void 0)];
                    case 4: return [2 /*return*/, _a.sent()];
                    case 5:
                        if (!(line instanceof Buffer)) return [3 /*break*/, 8];
                        return [4 /*yield*/, tslib_1.__await(line.toString('utf8'))];
                    case 6: return [4 /*yield*/, _a.sent()];
                    case 7:
                        _a.sent();
                        return [3 /*break*/, 9];
                    case 8: throw 'This should never happen';
                    case 9:
                        promises.pop();
                        resFns.pop();
                        return [3 /*break*/, 2];
                    case 10: return [3 /*break*/, 12];
                    case 11:
                        subscriptions.dispose();
                        return [7 /*endfinally*/];
                    case 12: return [2 /*return*/];
                }
            });
        });
    };
    return ChunkReader;
}());
/** Register multiple event listeners on a target, then unregister them all at once via dispose() */
var DisposableEventListener = /** @class */ (function () {
    function DisposableEventListener(emitter) {
        var _this = this;
        this.emitter = emitter;
        this.events = [];
        this.on = (function (evt, fn) {
            _this.emitter.on(evt, fn);
            _this.events.push({ evt: evt, fn: fn });
        });
    }
    DisposableEventListener.prototype.dispose = function () {
        var e_1, _a;
        try {
            for (var _b = tslib_1.__values(this.events), _c = _b.next(); !_c.done; _c = _b.next()) {
                var _d = _c.value, evt = _d.evt, fn = _d.fn;
                this.emitter.off(evt, fn);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        this.events.length = 0;
    };
    return DisposableEventListener;
}());
/**
 * NOTE: does *not* dispose of the chunkReader iterator.
 */
function getMessages(chunkReader) {
    return tslib_1.__asyncGenerator(this, arguments, function getMessages_1() {
        var next, messageLength, message, _a, _b, chunk, e_2_1;
        var e_2, _c;
        return tslib_1.__generator(this, function (_d) {
            switch (_d.label) {
                case 0:
                    if (!true) return [3 /*break*/, 18];
                    return [4 /*yield*/, tslib_1.__await(chunkReader.next())];
                case 1:
                    next = _d.sent();
                    if (!next.done) return [3 /*break*/, 3];
                    return [4 /*yield*/, tslib_1.__await(void 0)];
                case 2: return [2 /*return*/, _d.sent()];
                case 3:
                    messageLength = parseInt(next.value);
                    message = [];
                    if (!(messageLength !== 0)) return [3 /*break*/, 17];
                    _d.label = 4;
                case 4:
                    _d.trys.push([4, 11, 12, 17]);
                    _a = tslib_1.__asyncValues(noReturn(chunkReader));
                    _d.label = 5;
                case 5: return [4 /*yield*/, tslib_1.__await(_a.next())];
                case 6:
                    if (!(_b = _d.sent(), !_b.done)) return [3 /*break*/, 10];
                    chunk = _b.value;
                    message.push(chunk.toString());
                    if (!(message.length === messageLength)) return [3 /*break*/, 9];
                    return [4 /*yield*/, tslib_1.__await(message)];
                case 7: return [4 /*yield*/, _d.sent()];
                case 8:
                    _d.sent();
                    return [3 /*break*/, 10];
                case 9: return [3 /*break*/, 5];
                case 10: return [3 /*break*/, 17];
                case 11:
                    e_2_1 = _d.sent();
                    e_2 = { error: e_2_1 };
                    return [3 /*break*/, 17];
                case 12:
                    _d.trys.push([12, , 15, 16]);
                    if (!(_b && !_b.done && (_c = _a.return))) return [3 /*break*/, 14];
                    return [4 /*yield*/, tslib_1.__await(_c.call(_a))];
                case 13:
                    _d.sent();
                    _d.label = 14;
                case 14: return [3 /*break*/, 16];
                case 15:
                    if (e_2) throw e_2.error;
                    return [7 /*endfinally*/];
                case 16: return [7 /*endfinally*/];
                case 17: return [3 /*break*/, 0];
                case 18: return [2 /*return*/];
            }
        });
    });
}
/**
 * Wrap an async iterator so that .return() is not passed to the wrapped `iter`.
 * This means you can `for await (const foo of noReturn(iter))`, `break;` out of
 * the loop, and it won't tell `iter` to stop early.
 */
function noReturn(iter) {
    return tslib_1.__asyncGenerator(this, arguments, function noReturn_1() {
        var next, e_3;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!true) return [3 /*break*/, 9];
                    return [4 /*yield*/, tslib_1.__await(iter.next())];
                case 1:
                    next = _a.sent();
                    if (!next.done) return [3 /*break*/, 3];
                    return [4 /*yield*/, tslib_1.__await(void 0)];
                case 2: return [2 /*return*/, _a.sent()];
                case 3:
                    _a.trys.push([3, 6, , 8]);
                    return [4 /*yield*/, tslib_1.__await(next.value)];
                case 4: return [4 /*yield*/, _a.sent()];
                case 5:
                    _a.sent();
                    return [3 /*break*/, 8];
                case 6:
                    e_3 = _a.sent();
                    return [4 /*yield*/, tslib_1.__await(iter.throw(e_3))];
                case 7:
                    _a.sent();
                    return [3 /*break*/, 8];
                case 8: return [3 /*break*/, 0];
                case 9: return [2 /*return*/];
            }
        });
    });
}
exports.noReturn = noReturn;
/**
 * Encode an array of strings into a line of bash that does not contain any
 * newlines
 */
function bashEncodeStringArray(arr) {
    return shellQuote.quote(arr);
}
function start(opts) {
    var e_4, _a;
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var _b, input, _c, output, commands, lineReader, messageStream, messageStream_1, messageStream_1_1, message, _d, command, args, commandFn, response, result, e_5, encoded, e_4_1;
        return tslib_1.__generator(this, function (_e) {
            switch (_e.label) {
                case 0:
                    _b = opts.input, input = _b === void 0 ? fs_1.default.createReadStream('', { fd: 3 }) : _b, _c = opts.output, output = _c === void 0 ? fs_1.default.createWriteStream('', { fd: 4 }) : _c, commands = opts.commands;
                    lineReader = new ChunkReader(input);
                    messageStream = getMessages(lineReader.getChunks());
                    // TODO is this messy .resume() and .pause() stuff really necessary to avoid
                    // early data being lost?
                    lineReader.start();
                    input.resume();
                    _e.label = 1;
                case 1:
                    _e.trys.push([1, 10, 11, 16]);
                    messageStream_1 = tslib_1.__asyncValues(messageStream);
                    _e.label = 2;
                case 2: return [4 /*yield*/, messageStream_1.next()];
                case 3:
                    if (!(messageStream_1_1 = _e.sent(), !messageStream_1_1.done)) return [3 /*break*/, 9];
                    message = messageStream_1_1.value;
                    _d = tslib_1.__read(message), command = _d[0], args = _d.slice(1);
                    commandFn = commands[command];
                    assert_1.default(typeof commandFn === 'function', "\"" + command + "\" is not the name of an exported function.");
                    response = [0, '', ''];
                    _e.label = 4;
                case 4:
                    _e.trys.push([4, 6, , 7]);
                    return [4 /*yield*/, commandFn.apply(void 0, tslib_1.__spread(args))];
                case 5:
                    result = _e.sent();
                    assert_1.default(typeof result === 'string' || typeof result === 'number' || typeof result === 'boolean' || result == null, "Return value of \"" + command + "\" is not one of the supported return types.");
                    if (typeof result === 'boolean') {
                        response = [result ? 0 : 1, '', ''];
                    }
                    else {
                        response = [0, "" + result, ''];
                    }
                    return [3 /*break*/, 7];
                case 6:
                    e_5 = _e.sent();
                    response = [1, '', "" + e_5];
                    return [3 /*break*/, 7];
                case 7:
                    encoded = bashEncodeStringArray(response.map(function (v) { return "" + v; }));
                    // write encoded + trailing newline to output stream
                    output.write(encoded + '\n');
                    _e.label = 8;
                case 8: return [3 /*break*/, 2];
                case 9: return [3 /*break*/, 16];
                case 10:
                    e_4_1 = _e.sent();
                    e_4 = { error: e_4_1 };
                    return [3 /*break*/, 16];
                case 11:
                    _e.trys.push([11, , 14, 15]);
                    if (!(messageStream_1_1 && !messageStream_1_1.done && (_a = messageStream_1.return))) return [3 /*break*/, 13];
                    return [4 /*yield*/, _a.call(messageStream_1)];
                case 12:
                    _e.sent();
                    _e.label = 13;
                case 13: return [3 /*break*/, 15];
                case 14:
                    if (e_4) throw e_4.error;
                    return [7 /*endfinally*/];
                case 15: return [7 /*endfinally*/];
                case 16: return [2 /*return*/];
            }
        });
    });
}
exports.start = start;
function getBashStubs(bashCoprocName, commands) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var a, _a, _b, _c, name, fn;
        var e_6, _d;
        return tslib_1.__generator(this, function (_e) {
            a = "\n        function __node_ipc_" + bashCoprocName + "__() {\n            # Send to node as null-delimited sequence of messages\n            # first one is the number of following strings in the message\n            # next is a sequence of strings which will be parsed into an array\n            printf \"%s\\000\" \"${#@}\" \"$@\" >&\"${" + bashCoprocName + "[1]}\"\n            local returnValueRaw\n            local returnValues\n            IFS='' read -r -d $'\n' returnValueRaw <&\"${" + bashCoprocName + "[0]}\"\n    \n            # Parse returnValueRaw into array\n            eval \"returnValues=($returnValueRaw)\"\n\n            # first item in array is exit code (JS can send 1 for any exception, 0 for success)\n            # second item is return value for stdout\n            # third value is error message for stderr\n\n            # Write return value to stdout\n            printf '%s' \"${returnValues[1]}\"\n            # write error message to stderr\n            printf '%s' \"${returnValues[2]}\" >&2\n            # Return exit code\n            return \"${returnValues[0]}\"\n        }\n    ";
            try {
                for (_a = tslib_1.__values(Object.entries(commands)), _b = _a.next(); !_b.done; _b = _a.next()) {
                    _c = tslib_1.__read(_b.value, 2), name = _c[0], fn = _c[1];
                    a += "\n            function " + name + "() {\n                __node_ipc_" + bashCoprocName + "__ " + name + " \"$@\"\n            }\n        ";
                }
            }
            catch (e_6_1) { e_6 = { error: e_6_1 }; }
            finally {
                try {
                    if (_b && !_b.done && (_d = _a.return)) _d.call(_a);
                }
                finally { if (e_6) throw e_6.error; }
            }
            a += exports.STUBS_EOF_MARKER;
            return [2 /*return*/, a];
        });
    });
}
exports.getBashStubs = getBashStubs;
exports.STUBS_EOF_MARKER = '\nEOF\n';
//# sourceMappingURL=index.js.map